@@ -62,10 +62,26 @@ GType empathy_account_widget_get_type (void);
 GtkWidget *empathy_account_widget_get_widget (EmpathyAccountWidget *widget);
 
 EmpathyAccountWidget * empathy_account_widget_new_for_protocol (
-    const char *protocol,
     EmpathyAccountSettings *settings,
     gboolean simple);
 
+gboolean empathy_account_widget_contains_pending_changes
+    (EmpathyAccountWidget *widget);
+void empathy_account_widget_discard_pending_changes
+    (EmpathyAccountWidget *widget);
+
+gchar * empathy_account_widget_get_default_display_name (
+    EmpathyAccountWidget *widget);
+
+void empathy_account_widget_set_account_param (EmpathyAccountWidget *widget,
+    const gchar *account);
+
+void empathy_account_widget_set_password_param (EmpathyAccountWidget *self,
+    const gchar *password);
+
+/* protected methods */
+void empathy_account_widget_changed (EmpathyAccountWidget *widget);
+
 G_END_DECLS
 
 #endif /* __EMPATHY_ACCOUNT_WIDGET_H__ */
