@@ -45,7 +45,6 @@
 #include "empathy-dispatcher.h"
 #include "empathy-handler.h"
 #include "empathy-utils.h"
-#include "empathy-tube-handler.h"
 #include "empathy-tp-contact-factory.h"
 #include "empathy-chatroom-manager.h"
 #include "empathy-utils.h"
@@ -63,7 +62,6 @@ typedef struct
   GHashTable *connections;
   GHashTable *outstanding_classes_requests;
   gpointer token;
-  GSList *tubes;
 
   /* channels which the dispatcher is listening "invalidated" */
   GList *channels;
