@@ -385,7 +385,6 @@ main_window_error_display (EmpathyMainWindow *window,
 	GtkWidget *close_button;
 	GtkWidget *action_area;
 	GtkWidget *action_table;
-	GtkRcStyle *rc_style;
 	gchar     *str;
 	const gchar     *icon_name;
 
@@ -417,35 +416,29 @@ main_window_error_display (EmpathyMainWindow *window,
 
 	label = gtk_label_new (str);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
 	gtk_widget_show (label);
 	g_free (str);
 
 	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar));
 	gtk_box_pack_start (GTK_BOX (content_area), image, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (content_area), label, FALSE, FALSE, 0);
-
-	/* make small style for the buttons */
-	rc_style = gtk_rc_style_new ();
-	rc_style->xthickness = rc_style->ythickness = 4;
+	gtk_box_pack_start (GTK_BOX (content_area), label, TRUE, TRUE, 0);
 
 	image = gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON);
 	retry_button = gtk_button_new ();
 	gtk_button_set_image (GTK_BUTTON (retry_button), image);
-	gtk_widget_modify_style (retry_button, rc_style);
 	gtk_widget_set_tooltip_text (retry_button, _("Reconnect"));
 	gtk_widget_show (retry_button);
 
 	image = gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_BUTTON);
 	edit_button = gtk_button_new ();
 	gtk_button_set_image (GTK_BUTTON (edit_button), image);
-	gtk_widget_modify_style (edit_button, rc_style);
 	gtk_widget_set_tooltip_text (edit_button, _("Edit Account"));
 	gtk_widget_show (edit_button);
 
 	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
 	close_button = gtk_button_new ();
 	gtk_button_set_image (GTK_BUTTON (close_button), image);
-	gtk_widget_modify_style (close_button, rc_style);
 	gtk_widget_set_tooltip_text (close_button, _("Close"));
 	gtk_widget_show (close_button);
 
@@ -492,7 +485,6 @@ main_window_error_display (EmpathyMainWindow *window,
 
 	gtk_widget_show (window->errors_vbox);
 
-	g_object_unref (rc_style);
 	g_hash_table_insert (window->errors, g_object_ref (account), info_bar);
 }
 
@@ -818,9 +810,6 @@ main_window_view_contacts_list_size_cb (GtkRadioAction    *action,
 						     value == CONTACT_LIST_NORMAL_SIZE_WITH_AVATARS);
 	empathy_contact_list_store_set_is_compact (window->list_store,
 						   value == CONTACT_LIST_COMPACT_SIZE);
-
-	gtk_action_set_sensitive (GTK_ACTION (window->show_protocols),
-						value != CONTACT_LIST_COMPACT_SIZE );
 }
 
 static void main_window_notify_show_protocols_cb (EmpathyConf       *conf,
